/******************************************************************************
* Function name         :  system.c
* returns               :  
* Created by            :  Bryan Fletcher 
* Date Created          :  July 2, 2003
* Company				:  Memec
*
* Target                :  Memec Spartan-II LC board
*
* Description           :  Using loopback cables attached to JP[3:5], this program
*                       :  tests the connectivity between pairs of I/Os, writes to
*                       :  the 7-segment displays, toggles the LEDs, reads the DIP
*                       :  switches and PUSH2, printing the results to the UART.
* Notes                 :  
******************************************************************************/

#include "xgpio_l.h"
#include "xparameters.h"
#include "xuartlite_l.h"

#define true 1

//	The following function generates a variable delay
void _wait(loop_count)
{
	int	sum, data;

	sum	= 0;

	for (data = 0; data < loop_count; data++)
		sum = (data << 8);

	return;
}


//	The following function provides a simple print routine to print
//	characters to the UART. If stdio.h is included in the main program and
//	the UART is set to be STDIN/STDOUT, then a simple printf() function could
//	also be used.
void printf_uart(char *s)
{
	while (*s)
	{
		XUartLite_SendByte(XPAR_MYUART_BASEADDR,*s);
		while (XUartLite_mIsTransmitFull(XPAR_MYUART_BASEADDR) == true)
		{};
		s++;
	}
	return;
}


//	The following function displays an integer in binary format.
void printf_data(int data, int start_shift, int end_shift)
{
	int i, val;

	for (i = start_shift; i < end_shift; i++)
	{
		val = (data << i) & 0x80000000;
		if (val == 0x80000000)
			printf_uart("1");
		else
			printf_uart("0");
	}
	printf_uart("\n\r");
}

main()
{

	int loop_count;
	int	i, sw_input;

	int display_data[10] =	{0x01, 0x4f, 0x12, 0x06, 0x4c, 0x24, 0x20, 0x0f, 0x00, 0x04};

	int io_reg_0_read_data, io_reg_1_read_data, io_reg_2_read_data;
	int high_error, low_error, val;

	char *output_port[39];
	char *input_port[39];


// Test module JP3 connector pins
output_port[0]	= "164";
output_port[1]	= "166";
output_port[2]	= "168";
output_port[3]	= "173";
output_port[4]	= "175";
output_port[5]	= "178";
output_port[6]	= "180";
//output_port[7]	= "182"; //P182=GCLK causes EDK problems
output_port[8]	= "188";
output_port[9]	= "191";
output_port[10] = "193";
output_port[11] = "195";
output_port[12] = "200";

// Test module JP4 connector pins
output_port[13] = "109";
output_port[14] = "111";
output_port[15] = "113";
output_port[16] = "120";
output_port[17] = "122";
output_port[18] = "125";
output_port[19] = "129";
output_port[20] = "133";
output_port[21] = "136";
output_port[22] = "139";
output_port[23] = "141";
output_port[24] = "148";
output_port[25] = "150";

// Test module JP5 connector pins
output_port[26] = "202";
output_port[27] = "204";
output_port[28] = "206";
output_port[29] = "4  ";
output_port[30] = "6  ";
output_port[31] = "8  ";
output_port[32] = "10 ";
output_port[33] = "24 ";
output_port[34] = "29 ";
output_port[35] = "31 ";
output_port[36] = "34 ";
output_port[37] = "36 ";
output_port[38] = "41 ";

// Test module JP3 connector pins
input_port[0]	= "165";
input_port[1]	= "167";
input_port[2]	= "172";
input_port[3]	= "174";
input_port[4]	= "176";
input_port[5]	= "179";
input_port[6]	= "181";
//input_port[7]	= "187"; //P182=GCLK causes EDK problems
input_port[8]	= "189";
input_port[9]	= "192";
input_port[10]	= "194";
input_port[11]	= "199";
input_port[12]	= "201";

// Test module JP4 connector pins
input_port[13]	= "110";
input_port[14]	= "112";
input_port[15]	= "114";
input_port[16]	= "121";
input_port[17]	= "123";
input_port[18]	= "127";
input_port[19]	= "132";
input_port[20]	= "134";
input_port[21]	= "138";
input_port[22]	= "140";
input_port[23]	= "147";
input_port[24]	= "149";
input_port[25]	= "151";

// Test module JP5 connector pins
input_port[26]	= "203";
input_port[27]	= "205";
input_port[28]	= "3  ";
input_port[29]	= "5  ";
input_port[30]	= "7  ";
input_port[31]	= "9  ";
input_port[32]	= "14 ";
input_port[33]	= "27 ";
input_port[34]	= "30 ";
input_port[35]  = "33 ";
input_port[36]  = "35 ";
input_port[37]  = "37 ";
input_port[38]  = "42 ";

printf_uart("\f*** Memec Board S2LC Functional Test ***\n\r\n");

	printf_uart("Init...");

// Initialize all GPIO ports and set the direction
// 0 = output, 1 = input
	XGpio_mSetDataDirection(XPAR_SEG7_GPIO_A_BASEADDR, 0x00);
	XGpio_mSetDataDirection(XPAR_SEG7_GPIO_B_BASEADDR, 0x00);
	XGpio_mSetDataDirection(XPAR_LED_GPIO_BASEADDR, 0x0);
	XGpio_mSetDataDirection(XPAR_DIP_GPIO_BASEADDR, 0xf);
	XGpio_mSetDataDirection(XPAR_PUSH_GPIO_BASEADDR, 0x3);
	XGpio_mSetDataDirection(XPAR_JP3OUT_GPIO_BASEADDR, 0x0000);
	XGpio_mSetDataDirection(XPAR_JP3IN_GPIO_BASEADDR, 0xffff);
	XGpio_mSetDataDirection(XPAR_JP4OUT_GPIO_BASEADDR, 0x0000);
	XGpio_mSetDataDirection(XPAR_JP4IN_GPIO_BASEADDR, 0xffff);
	XGpio_mSetDataDirection(XPAR_JP5OUT_GPIO_BASEADDR, 0x0000);
	XGpio_mSetDataDirection(XPAR_JP5IN_GPIO_BASEADDR, 0xffff);

	printf_uart("Done\n\r\n");

// User I/O loopback test.
	printf_uart("I/O Test...");
	high_error	= 0;
	low_error	= 0;

	
//	Forcing user I/O output pins to "1" and reading them back
	XGpio_mSetDataReg(XPAR_JP3OUT_GPIO_BASEADDR, 0x1FFF);
	XGpio_mSetDataReg(XPAR_JP4OUT_GPIO_BASEADDR, 0x1FFF);
	XGpio_mSetDataReg(XPAR_JP5OUT_GPIO_BASEADDR, 0x1FFF);


	io_reg_0_read_data = XGpio_mGetDataReg(XPAR_JP3IN_GPIO_BASEADDR);
	io_reg_1_read_data = XGpio_mGetDataReg(XPAR_JP4IN_GPIO_BASEADDR);
	io_reg_2_read_data = XGpio_mGetDataReg(XPAR_JP5IN_GPIO_BASEADDR);



	for (i = 0; i < 13; i++) {
			// Check JP3 connections
			if(i!=7)  //skipping P182 GCLK
			{
			  val = (io_reg_0_read_data << i) & 0x00001000;
			  if (val != 0x00001000)
			  {
					printf_uart("\n\rP");
					printf_uart(output_port[i]);
					printf_uart(" or P");
					printf_uart(input_port[i]);
					printf_uart(" = GND");
					high_error = high_error + 1;
			  }

            }

			// Check JP4 connections
			val = (io_reg_1_read_data << i) & 0x00001000;
			if (val != 0x00001000)
			{
					printf_uart("\n\rP");
					printf_uart(output_port[i+13]);
					printf_uart(" or P");
					printf_uart(input_port[i+13]);
					printf_uart(" = GND");
					high_error = high_error + 1;
			}


			// Check JP5 connections
			val = (io_reg_2_read_data << i) & 0x00001000;
			if (val != 0x00001000)
			{
					printf_uart("\n\rP");
					printf_uart(output_port[i+26]);
					printf_uart("or P");
					printf_uart(input_port[i+26]);
					printf_uart(" = GND");
					high_error = high_error + 1;
			}

	}



//	Forcing User I/O output pins to "0" and reading them back

	XGpio_mSetDataReg(XPAR_JP3OUT_GPIO_BASEADDR, 0x0000);
	XGpio_mSetDataReg(XPAR_JP4OUT_GPIO_BASEADDR, 0x0000);
	XGpio_mSetDataReg(XPAR_JP5OUT_GPIO_BASEADDR, 0x0000);


	io_reg_0_read_data = XGpio_mGetDataReg(XPAR_JP3IN_GPIO_BASEADDR);
	io_reg_1_read_data = XGpio_mGetDataReg(XPAR_JP4IN_GPIO_BASEADDR);
	io_reg_2_read_data = XGpio_mGetDataReg(XPAR_JP5IN_GPIO_BASEADDR);


	for (i = 0; i < 13; i++) {
			if(i!=7)	//skipping P182 GCLK
			{
			  // Check JP3 connections
			  val = (io_reg_0_read_data << i) & 0x00001000;
			  if (val == 0x00001000)
			  {
					printf_uart("\n\rP");
					printf_uart(output_port[i]);
					printf_uart(" or P");
					printf_uart(input_port[i]);
					printf_uart(" = open or VCC");
				  	low_error = low_error + 1;
			  }
			}

			  // Check JP4 connections
			  val = (io_reg_1_read_data << i) & 0x00001000;
			  if (val == 0x00001000)
			  {
					printf_uart("\n\rP");
					printf_uart(output_port[i+13]);
					printf_uart(" or P");
					printf_uart(input_port[i+13]);
					printf_uart(" = open or VCC");
				  	low_error = low_error + 1;
			  }


			  // Check JP5 connections
			  val = (io_reg_2_read_data << i) & 0x00001000;
			  if (val == 0x00001000)
			  {
					printf_uart("\n\rP");
					printf_uart(output_port[i+26]);
					printf_uart(" or P");
					printf_uart(input_port[i+26]);
					printf_uart(" = open or VCC");
				  	low_error = low_error + 1;
			  }

	}

if ((high_error == 0) & (low_error == 0))
	printf_uart("Passed\n\r");
else
	printf_uart("\n\r");


//	Writing 0-9 on each of two 7-Segment LEDs, lighting two LEDs with LSBs of counter,
//  reading the DIP Switch and PUSH2 whose values are displayed on the UART.

//	When debugging the code, please set a break point anywhere in the following
//	"while" loop to allow the xmd debugger to respond to the commands.

	printf_uart("\nLED/7-seg/Switch/Button Test...\n\r");

	while (1)
	{
		for (i = 0; i < 10; i++)
		{
			XGpio_mSetDataReg(XPAR_SEG7_GPIO_B_BASEADDR, display_data[i]);
			XGpio_mSetDataReg(XPAR_SEG7_GPIO_A_BASEADDR, display_data[i]);
				
			sw_input = ~(XGpio_mGetDataReg(XPAR_DIP_GPIO_BASEADDR));
			printf_uart("DIPs(1-4)=");
			printf_data(sw_input, 28, 32);

			sw_input = XGpio_mGetDataReg(XPAR_PUSH_GPIO_BASEADDR);
			printf_uart("PUSH2 ");
			if (sw_input)
				printf_uart("released\n\r");
			else
				printf_uart("pushed\n\r");

			XGpio_mSetDataReg(XPAR_LED_GPIO_BASEADDR, i);
			_wait(2000000);
		}
	}

}